<?php
/**
 *  Fluistr Authentication pluggable
 *
 *  @package fluistr-athentication
 *  @version 1.0.1
 *  @subpackage fluistr-login
 */

/**
 * Handles sending password retrieval email to user.
 *
 * @since 2.5.0
 *
 * @return bool|WP_Error True: when finish. WP_Error on error
 */

if ( ! function_exists( 'fluistr_retrieve_challenge' ) ) :
	function fluistr_retrieve_challenge( $user_data, $type = 'setup' ) {
		$errors = new WP_Error();

		/**
		 * Fires before errors are returned from a password reset request.
		 *
		 * @since 2.1.0
		 * @since 4.4.0 Added the `$errors` parameter.
		 *
		 * @param WP_Error $errors A WP_Error object containing any errors generated
		 *                         by using invalid credentials.
		 */
		do_action( 'lostpassword_post', $errors );

		if ( $errors->has_errors() ) {
			return $errors;
		}

		if ( ! $user_data ) {
			$errors->add( 'invalidcombo', __( '<strong>ERROR</strong>: There is no account with that username or email address.' ) );
			return $errors;
		}

		// Redefining user_login ensures we return the right case in the email.
		$user_login = $user_data->user_login;
		$user_email = $user_data->user_email;
		$key        = get_password_reset_key( $user_data );

		if ( is_wp_error( $key ) ) {
			return $key;
		}

		if ( is_multisite() ) {
			$site_name = get_network()->site_name;
		} else {
			/*
			* The blogname option is escaped with esc_html on the way into the database
			* in sanitize_option we want to reverse this for the plain text arena of emails.
			*/
			$site_name = wp_specialchars_decode( get_option( 'blogname' ), ENT_QUOTES );
		}

		$information = WPFluistrAuth::get_option( 'mail_login_info' );

		if ( 'magiclink' === $type ) {
			$title    = sprintf( __( 'Login with Magic link [%s]' ), $site_name );
			$message  = WPFluistrAuth::get_fluistr_html_template( 'mail-magiclink' );
			$message .= WPFluistrAuth::get_fluistr_html_template( 'mail-footer' );
		} else {
			$title    = sprintf( __( '[%s] Login Details' ), $site_name );
			$message  = WPFluistrAuth::get_fluistr_html_template( 'mail-connect' );
			$message .= WPFluistrAuth::get_fluistr_html_template( 'mail-footer' );
		}

		if ( empty( $message ) ) {
			$errors->add( 'mailerror', __( '<strong>ERROR</strong>: The mail template file is missing.' ) );
			return $errors;
		}

		$link    = FluistrLoginController::fluistr_login_url() . 'action=activate&key=' . $key . '&login=' . rawurlencode( $user_login );
		$message = str_replace( '*|LOGIN_INFORMATION|*', nl2br( $information ), $message );
		$message = str_replace( '*|ASSETS_URL|*', plugin_dir_url( __FILE__ ) . 'assets', $message );
		$message = str_replace( '*|BLOGNAME|*', $site_name, $message );
		$message = str_replace( '*|USER_LOGIN|*', $user_login, $message );
		$message = str_replace( '*|USER_EMAIL|*', $user_email, $message );
		$message = str_replace( '*|LINK|*', $link, $message );

		/* translators: Password reset email subject. %s: Site name */

		/**
		 * Filters the subject of the password reset email.
		 *
		 * @since 2.8.0
		 * @since 4.4.0 Added the `$user_login` and `$user_data` parameters.
		 *
		 * @param string  $title      Default email title.
		 * @param string  $user_login The username for the user.
		 * @param WP_User $user_data  WP_User object.
		 */
		$title = apply_filters( 'retrieve_password_title', $title, $user_login, $user_data );

		/**
		 * Filters the message body of the password reset mail.
		 *
		 * If the filtered message is empty, the password reset email will not be sent.
		 *
		 * @since 2.8.0
		 * @since 4.1.0 Added `$user_login` and `$user_data` parameters.
		 *
		 * @param string  $message    Default mail message.
		 * @param string  $key        The activation key.
		 * @param string  $user_login The username for the user.
		 * @param WP_User $user_data  WP_User object.
		 */
		$message = apply_filters( 'retrieve_password_message', $message, $key, $user_login, $user_data );

		// Function to change sender name.
		function wpb_sender_name( $original_email_from ) {
			return $original_email_from . ' [Fluistr]';
		}

		// Hooking up our functions to WordPress filters.
		add_filter( 'wp_mail_from_name', 'wpb_sender_name' );

		$headers = array( 'Content-Type: text/html; charset=UTF-8' );

		if ( $message && ! wp_mail( $user_email, wp_specialchars_decode( $title ), $message, $headers ) ) {
			$errors->add( 'mailerror', __( '<strong>ERROR</strong>: Possible reason: your host may have disabled the mail() function.' ) );
			return $errors;
		}

		return true;
	}
endif;
